from pulumi_aws import get_caller_identity, iam

from cloudlib.modules.s3 import BucketPolicyFactory


class CloudFrontBucketAccessLogPolicyFactory(BucketPolicyFactory):
    def __init__(self, accounts=None):
        self.accounts = accounts
        super().__init__()

    def create(self, bucket):
        identity = get_caller_identity()
        accounts = self.accounts or [identity.account_id]

        statements = []

        for account_id in accounts:
            statements.append(
                iam.GetPolicyDocumentStatementArgs(
                    sid=f"PolicyForCloudLogs_{account_id}",
                    effect="Allow",
                    principals=[
                        iam.GetPolicyDocumentStatementPrincipalArgs(
                            type="Service", identifiers=["cloudfront.amazonaws.com"]
                        )
                    ],
                    actions=["s3:PutObject"],
                    resources=[f"arn:aws:s3:::{bucket}/*"],
                    conditions=[
                        iam.GetPolicyDocumentStatementConditionArgs(
                            test="StringEquals",
                            variable="aws:SourceAccount",
                            values=[account_id],
                        ),
                    ],
                )
            )

        return super().create(bucket, extra_statements=statements)
