from typing import Optional, Mapping
import pulumi

from cloudlib.modules.resource import BaseFactory
from pulumi_aws import guardduty


class MemberFactory(BaseFactory):
    def __init__(
        self,
        opts: Optional[pulumi.ResourceOptions] = None,
        account_id: Optional[pulumi.Input[str]] = None,
        detector_id: Optional[pulumi.Input[str]] = None,
        disable_email_notification: Optional[pulumi.Input[bool]] = None,
        email: Optional[pulumi.Input[str]] = None,
        invitation_message: Optional[pulumi.Input[str]] = None,
        invite: Optional[pulumi.Input[bool]] = None,
    ):
        super().__init__(
            resource_name=f"invitation_{account_id}",
            opts=opts,
            account_id=account_id,
            detector_id=detector_id,
            disable_email_notification=disable_email_notification,
            email=email,
            invitation_message=invitation_message,
            invite=invite,
        )

    def create(self):
        return self._create(guardduty.Member)


class DetectorFactory(BaseFactory):
    def __init__(
        self,
        opts: Optional[pulumi.ResourceOptions] = None,
        datasources: Optional[pulumi.Input[pulumi.InputType["DetectorDatasourcesArgs"]]] = None,
        enable: Optional[pulumi.Input[bool]] = None,
        finding_publishing_frequency: Optional[pulumi.Input[str]] = None,
        tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
    ):
        super().__init__(
            resource_name="detector",
            opts=opts,
            datasources=datasources,
            enable=enable,
            finding_publishing_frequency=finding_publishing_frequency,
            tags=tags,
        )
        self.members = []

    def member(self, account):
        self.members.append(
            MemberFactory(account_id=account.id, email=account.email, invite=True, detector_id=self.ref("id"))
        )

    def create(self):
        self.setdefault(
            "datasources",
            guardduty.DetectorDatasourcesArgs(
                malware_protection=guardduty.DetectorDatasourcesMalwareProtectionArgs(
                    scan_ec2_instance_with_findings=guardduty.DetectorDatasourcesMalwareProtectionScanEc2InstanceWithFindingsArgs(
                        ebs_volumes=guardduty.DetectorDatasourcesMalwareProtectionScanEc2InstanceWithFindingsEbsVolumesArgs(
                            enable=True,
                        ),
                    ),
                ),
                s3_logs=guardduty.DetectorDatasourcesS3LogsArgs(
                    enable=True,
                ),
            ),
        )
        self.setdefault("enable", True)

        detector = self._create(guardduty.Detector)

        for member in self.members:
            member.create()

        return detector
