from cloudlib.modules.s3 import BucketPolicyFactory
from pulumi_aws import iam


class LogsS3PolicyFactory(BucketPolicyFactory):
    def __init__(self, bucket_name, accounts):
        self.bucket_name = bucket_name
        self.accounts = accounts
        super().__init__()

    def create(self, bucket):
        statements = []

        for account in self.accounts:
            statements.append(
                iam.GetPolicyDocumentStatementArgs(
                    sid=f"AWSConfigBucketPermissionsCheck_{account}",
                    effect="Allow",
                    principals=[
                        iam.GetPolicyDocumentStatementPrincipalArgs(
                            type="Service",
                            identifiers=["delivery.logs.amazonaws.com"],
                        )
                    ],
                    actions=["s3:GetBucketAcl", "s3:ListBucket"],
                    resources=[f"arn:aws:s3:::{bucket}"],
                    conditions=[
                        iam.GetPolicyDocumentStatementConditionArgs(
                            test="StringEquals", variable="AWS:SourceAccount", values=[account]
                        ),
                    ],
                )
            )

            statements.append(
                iam.GetPolicyDocumentStatementArgs(
                    sid=f"AWSLogDeliveryWrite_{account}",
                    effect="Allow",
                    principals=[
                        iam.GetPolicyDocumentStatementPrincipalArgs(
                            type="Service",
                            identifiers=["delivery.logs.amazonaws.com"],
                        )
                    ],
                    actions=["s3:PutObject"],
                    resources=[
                        f"arn:aws:s3:::{bucket}/AWSLogs/{account}",
                        f"arn:aws:s3:::{bucket}/AWSLogs/{account}/*",
                    ],
                    conditions=[
                        iam.GetPolicyDocumentStatementConditionArgs(
                            test="StringEquals", variable="s3:x-amz-acl", values=["bucket-owner-full-control"]
                        ),
                        iam.GetPolicyDocumentStatementConditionArgs(
                            test="StringEquals", variable="AWS:SourceAccount", values=[account]
                        ),
                    ],
                ),
            )

        return super().create(bucket, extra_statements=statements)
