from typing import Optional, Sequence, Union, Mapping, overload

import pulumi
from pulumi_aws import ec2

from cloudlib.modules.resource import BaseResourceMixin, BaseFactory
from pulumi_aws.ec2 import SecurityGroupArgs


class InstanceFactory(BaseFactory):
    def __init__(
        self,
        name: str,
        opts: Optional[pulumi.ResourceOptions] = None,
        ami: Optional[pulumi.Input[str]] = None,
        associate_public_ip_address: Optional[pulumi.Input[bool]] = None,
        availability_zone: Optional[pulumi.Input[str]] = None,
        capacity_reservation_specification: Optional[
            pulumi.Input[pulumi.InputType["InstanceCapacityReservationSpecificationArgs"]]
        ] = None,
        cpu_core_count: Optional[pulumi.Input[int]] = None,
        cpu_options: Optional[pulumi.Input[pulumi.InputType["InstanceCpuOptionsArgs"]]] = None,
        cpu_threads_per_core: Optional[pulumi.Input[int]] = None,
        credit_specification: Optional[pulumi.Input[pulumi.InputType["InstanceCreditSpecificationArgs"]]] = None,
        disable_api_stop: Optional[pulumi.Input[bool]] = None,
        disable_api_termination: Optional[pulumi.Input[bool]] = None,
        ebs_block_devices: Optional[
            pulumi.Input[Sequence[pulumi.Input[pulumi.InputType["InstanceEbsBlockDeviceArgs"]]]]
        ] = None,
        ebs_optimized: Optional[pulumi.Input[bool]] = None,
        enclave_options: Optional[pulumi.Input[pulumi.InputType["InstanceEnclaveOptionsArgs"]]] = None,
        ephemeral_block_devices: Optional[
            pulumi.Input[Sequence[pulumi.Input[pulumi.InputType["InstanceEphemeralBlockDeviceArgs"]]]]
        ] = None,
        get_password_data: Optional[pulumi.Input[bool]] = None,
        hibernation: Optional[pulumi.Input[bool]] = None,
        host_id: Optional[pulumi.Input[str]] = None,
        host_resource_group_arn: Optional[pulumi.Input[str]] = None,
        iam_instance_profile: Optional[pulumi.Input[str]] = None,
        instance_initiated_shutdown_behavior: Optional[pulumi.Input[str]] = None,
        instance_market_options: Optional[pulumi.Input[pulumi.InputType["InstanceInstanceMarketOptionsArgs"]]] = None,
        instance_type: Optional[pulumi.Input[Union[str, "InstanceType"]]] = None,
        ipv6_address_count: Optional[pulumi.Input[int]] = None,
        ipv6_addresses: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
        key_name: Optional[pulumi.Input[str]] = None,
        launch_template: Optional[pulumi.Input[pulumi.InputType["InstanceLaunchTemplateArgs"]]] = None,
        maintenance_options: Optional[pulumi.Input[pulumi.InputType["InstanceMaintenanceOptionsArgs"]]] = None,
        metadata_options: Optional[pulumi.Input[pulumi.InputType["InstanceMetadataOptionsArgs"]]] = None,
        monitoring: Optional[pulumi.Input[bool]] = None,
        network_interfaces: Optional[
            pulumi.Input[Sequence[pulumi.Input[pulumi.InputType["InstanceNetworkInterfaceArgs"]]]]
        ] = None,
        placement_group: Optional[pulumi.Input[str]] = None,
        placement_partition_number: Optional[pulumi.Input[int]] = None,
        private_dns_name_options: Optional[pulumi.Input[pulumi.InputType["InstancePrivateDnsNameOptionsArgs"]]] = None,
        private_ip: Optional[pulumi.Input[str]] = None,
        root_block_device: Optional[pulumi.Input[pulumi.InputType["InstanceRootBlockDeviceArgs"]]] = None,
        secondary_private_ips: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
        security_groups: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
        source_dest_check: Optional[pulumi.Input[bool]] = None,
        subnet_id: Optional[pulumi.Input[str]] = None,
        tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
        tenancy: Optional[pulumi.Input[Union[str, "Tenancy"]]] = None,
        user_data: Optional[pulumi.Input[str]] = None,
        user_data_base64: Optional[pulumi.Input[str]] = None,
        user_data_replace_on_change: Optional[pulumi.Input[bool]] = None,
        volume_tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
        vpc_security_group_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
        __props__=None,
    ):
        super().__init__(
            name,
            opts=opts,
            ami=ami,
            associate_public_ip_address=associate_public_ip_address,
            availability_zone=availability_zone,
            capacity_reservation_specification=capacity_reservation_specification,
            cpu_core_count=cpu_core_count,
            cpu_options=cpu_options,
            cpu_threads_per_core=cpu_threads_per_core,
            credit_specification=credit_specification,
            disable_api_stop=disable_api_stop,
            disable_api_termination=disable_api_termination,
            ebs_block_devices=ebs_block_devices,
            ebs_optimized=ebs_optimized,
            enclave_options=enclave_options,
            ephemeral_block_devices=ephemeral_block_devices,
            get_password_data=get_password_data,
            hibernation=hibernation,
            host_id=host_id,
            host_resource_group_arn=host_resource_group_arn,
            iam_instance_profile=iam_instance_profile,
            instance_initiated_shutdown_behavior=instance_initiated_shutdown_behavior,
            instance_market_options=instance_market_options,
            instance_type=instance_type,
            ipv6_address_count=ipv6_address_count,
            ipv6_addresses=ipv6_addresses,
            key_name=key_name,
            launch_template=launch_template,
            maintenance_options=maintenance_options,
            metadata_options=metadata_options,
            monitoring=monitoring,
            network_interfaces=network_interfaces,
            placement_group=placement_group,
            placement_partition_number=placement_partition_number,
            private_dns_name_options=private_dns_name_options,
            private_ip=private_ip,
            root_block_device=root_block_device,
            secondary_private_ips=secondary_private_ips,
            security_groups=security_groups,
            source_dest_check=source_dest_check,
            subnet_id=subnet_id,
            tags=tags,
            tenancy=tenancy,
            user_data=user_data,
            user_data_base64=user_data_base64,
            user_data_replace_on_change=user_data_replace_on_change,
            volume_tags=volume_tags,
            vpc_security_group_ids=vpc_security_group_ids,
            __props__=__props__,
        )

    def create(self):
        self.setdefault("metadata_options", ec2.InstanceMetadataOptionsArgs(http_tokens="required"))

        return self._create(ec2.Instance)


class Instance(ec2.Instance, BaseResourceMixin):
    def __init__(
        self,
        name: str,
        opts: Optional[pulumi.ResourceOptions] = None,
        ami: Optional[pulumi.Input[str]] = None,
        associate_public_ip_address: Optional[pulumi.Input[bool]] = None,
        availability_zone: Optional[pulumi.Input[str]] = None,
        capacity_reservation_specification: Optional[
            pulumi.Input[pulumi.InputType["InstanceCapacityReservationSpecificationArgs"]]
        ] = None,
        cpu_core_count: Optional[pulumi.Input[int]] = None,
        cpu_options: Optional[pulumi.Input[pulumi.InputType["InstanceCpuOptionsArgs"]]] = None,
        cpu_threads_per_core: Optional[pulumi.Input[int]] = None,
        credit_specification: Optional[pulumi.Input[pulumi.InputType["InstanceCreditSpecificationArgs"]]] = None,
        disable_api_stop: Optional[pulumi.Input[bool]] = None,
        disable_api_termination: Optional[pulumi.Input[bool]] = None,
        ebs_block_devices: Optional[
            pulumi.Input[Sequence[pulumi.Input[pulumi.InputType["InstanceEbsBlockDeviceArgs"]]]]
        ] = None,
        ebs_optimized: Optional[pulumi.Input[bool]] = None,
        enclave_options: Optional[pulumi.Input[pulumi.InputType["InstanceEnclaveOptionsArgs"]]] = None,
        ephemeral_block_devices: Optional[
            pulumi.Input[Sequence[pulumi.Input[pulumi.InputType["InstanceEphemeralBlockDeviceArgs"]]]]
        ] = None,
        get_password_data: Optional[pulumi.Input[bool]] = None,
        hibernation: Optional[pulumi.Input[bool]] = None,
        host_id: Optional[pulumi.Input[str]] = None,
        host_resource_group_arn: Optional[pulumi.Input[str]] = None,
        iam_instance_profile: Optional[pulumi.Input[str]] = None,
        instance_initiated_shutdown_behavior: Optional[pulumi.Input[str]] = None,
        instance_market_options: Optional[pulumi.Input[pulumi.InputType["InstanceInstanceMarketOptionsArgs"]]] = None,
        instance_type: Optional[pulumi.Input[Union[str, "InstanceType"]]] = None,
        ipv6_address_count: Optional[pulumi.Input[int]] = None,
        ipv6_addresses: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
        key_name: Optional[pulumi.Input[str]] = None,
        launch_template: Optional[pulumi.Input[pulumi.InputType["InstanceLaunchTemplateArgs"]]] = None,
        maintenance_options: Optional[pulumi.Input[pulumi.InputType["InstanceMaintenanceOptionsArgs"]]] = None,
        metadata_options: Optional[pulumi.Input[pulumi.InputType["InstanceMetadataOptionsArgs"]]] = None,
        monitoring: Optional[pulumi.Input[bool]] = None,
        network_interfaces: Optional[
            pulumi.Input[Sequence[pulumi.Input[pulumi.InputType["InstanceNetworkInterfaceArgs"]]]]
        ] = None,
        placement_group: Optional[pulumi.Input[str]] = None,
        placement_partition_number: Optional[pulumi.Input[int]] = None,
        private_dns_name_options: Optional[pulumi.Input[pulumi.InputType["InstancePrivateDnsNameOptionsArgs"]]] = None,
        private_ip: Optional[pulumi.Input[str]] = None,
        root_block_device: Optional[pulumi.Input[pulumi.InputType["InstanceRootBlockDeviceArgs"]]] = None,
        secondary_private_ips: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
        security_groups: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
        source_dest_check: Optional[pulumi.Input[bool]] = None,
        subnet_id: Optional[pulumi.Input[str]] = None,
        tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
        tenancy: Optional[pulumi.Input[Union[str, "Tenancy"]]] = None,
        user_data: Optional[pulumi.Input[str]] = None,
        user_data_base64: Optional[pulumi.Input[str]] = None,
        user_data_replace_on_change: Optional[pulumi.Input[bool]] = None,
        volume_tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
        vpc_security_group_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
        __props__=None,
    ):
        BaseResourceMixin.__init__(
            self,
            parent=ec2.Instance,
            resource_name=name,
            opts=opts,
            ami=ami,
            associate_public_ip_address=associate_public_ip_address,
            availability_zone=availability_zone,
            capacity_reservation_specification=capacity_reservation_specification,
            cpu_core_count=cpu_core_count,
            cpu_options=cpu_options,
            cpu_threads_per_core=cpu_threads_per_core,
            credit_specification=credit_specification,
            disable_api_stop=disable_api_stop,
            disable_api_termination=disable_api_termination,
            ebs_block_devices=ebs_block_devices,
            ebs_optimized=ebs_optimized,
            enclave_options=enclave_options,
            ephemeral_block_devices=ephemeral_block_devices,
            get_password_data=get_password_data,
            hibernation=hibernation,
            host_id=host_id,
            host_resource_group_arn=host_resource_group_arn,
            iam_instance_profile=iam_instance_profile,
            instance_initiated_shutdown_behavior=instance_initiated_shutdown_behavior,
            instance_market_options=instance_market_options,
            instance_type=instance_type,
            ipv6_address_count=ipv6_address_count,
            ipv6_addresses=ipv6_addresses,
            key_name=key_name,
            launch_template=launch_template,
            maintenance_options=maintenance_options,
            metadata_options=metadata_options,
            monitoring=monitoring,
            network_interfaces=network_interfaces,
            placement_group=placement_group,
            placement_partition_number=placement_partition_number,
            private_dns_name_options=private_dns_name_options,
            private_ip=private_ip,
            root_block_device=root_block_device,
            secondary_private_ips=secondary_private_ips,
            security_groups=security_groups,
            source_dest_check=source_dest_check,
            subnet_id=subnet_id,
            tags=tags,
            tenancy=tenancy,
            user_data=user_data,
            user_data_base64=user_data_base64,
            user_data_replace_on_change=user_data_replace_on_change,
            volume_tags=volume_tags,
            vpc_security_group_ids=vpc_security_group_ids,
            __props__=__props__,
        )


class SecurityGroup(ec2.SecurityGroup, BaseResourceMixin):
    def __init__(
        self,
        name: str,
        opts: Optional[pulumi.ResourceOptions] = None,
        description: Optional[pulumi.Input[str]] = None,
        egress: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType["SecurityGroupEgressArgs"]]]]] = None,
        ingress: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType["SecurityGroupIngressArgs"]]]]] = None,
        name_prefix: Optional[pulumi.Input[str]] = None,
        revoke_rules_on_delete: Optional[pulumi.Input[bool]] = None,
        tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
        vpc_id: Optional[pulumi.Input[str]] = None,
        __props__=None,
    ):
        BaseResourceMixin.__init__(
            self,
            parent=ec2.SecurityGroup,
            resource_name=name,
            opts=opts,
            description=description,
            egress=egress,
            ingress=ingress,
            name=name,
            name_prefix=name_prefix,
            revoke_rules_on_delete=revoke_rules_on_delete,
            tags=tags,
            vpc_id=vpc_id,
            __props__=__props__,
        )
