from typing import Optional, Sequence, Mapping

import pulumi
from pulumi_aws import ram

from cloudlib.modules.resource import BaseResourceMixin
from cloudlib.utils.names import IdentifierV2


class ResourceShare(ram.ResourceShare, BaseResourceMixin):
    def __init__(
        self,
        name: str,
        opts: Optional[pulumi.ResourceOptions] = None,
        allow_external_principals: Optional[pulumi.Input[bool]] = None,
        permission_arns: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
        tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
        __props__=None,
        **kwargs
    ):
        BaseResourceMixin.__init__(
            self,
            parent=ram.ResourceShare,
            resource_name=name,
            name=IdentifierV2(name).spinal,
            opts=opts,
            allow_external_principals=allow_external_principals,
            permission_arns=permission_arns,
            tags=tags,
            __props__=__props__,
            **kwargs
        )


class ResourceAssociation(ram.ResourceAssociation, BaseResourceMixin):
    def __init__(
        self,
        name: str,
        opts: Optional[pulumi.ResourceOptions] = None,
        resource_arn: Optional[pulumi.Input[str]] = None,
        resource_share_arn: Optional[pulumi.Input[str]] = None,
        __props__=None,
    ):
        BaseResourceMixin.__init__(
            self,
            parent=ram.ResourceAssociation,
            resource_name=name,
            opts=opts,
            resource_arn=resource_arn,
            resource_share_arn=resource_share_arn,
            __props__=__props__,
        )


class PrincipalAssociation(ram.PrincipalAssociation, BaseResourceMixin):
    def __init__(
        self,
        name: str,
        opts: Optional[pulumi.ResourceOptions] = None,
        principal: Optional[pulumi.Input[str]] = None,
        resource_share_arn: Optional[pulumi.Input[str]] = None,
        __props__=None,
    ):
        BaseResourceMixin.__init__(
            self,
            parent=ram.PrincipalAssociation,
            resource_name=name,
            opts=opts,
            principal=principal,
            resource_share_arn=resource_share_arn,
        )
