from typing import Optional, Sequence, Union, Mapping

import pulumi
from pulumi_aws import rds

from cloudlib.modules.ec2 import SecurityGroup
from cloudlib.modules.kms import Key
from cloudlib.modules.resource import BaseResourceMixin


class Cluster(rds.Cluster, BaseResourceMixin):
    def __init__(
        self,
        name: str,
        opts: Optional[pulumi.ResourceOptions] = None,
        allocated_storage: Optional[pulumi.Input[int]] = None,
        allow_major_version_upgrade: Optional[pulumi.Input[bool]] = None,
        apply_immediately: Optional[pulumi.Input[bool]] = None,
        availability_zones: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
        backtrack_window: Optional[pulumi.Input[int]] = None,
        backup_retention_period: Optional[pulumi.Input[int]] = None,
        cluster_identifier: Optional[pulumi.Input[str]] = None,
        cluster_identifier_prefix: Optional[pulumi.Input[str]] = None,
        cluster_members: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
        copy_tags_to_snapshot: Optional[pulumi.Input[bool]] = None,
        database_name: Optional[pulumi.Input[str]] = None,
        db_cluster_instance_class: Optional[pulumi.Input[str]] = None,
        db_cluster_parameter_group_name: Optional[pulumi.Input[str]] = None,
        db_instance_parameter_group_name: Optional[pulumi.Input[str]] = None,
        db_subnet_group_name: Optional[pulumi.Input[str]] = None,
        deletion_protection: Optional[pulumi.Input[bool]] = None,
        enable_global_write_forwarding: Optional[pulumi.Input[bool]] = None,
        enable_http_endpoint: Optional[pulumi.Input[bool]] = None,
        enabled_cloudwatch_logs_exports: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
        engine: Optional[pulumi.Input[Union[str, "EngineType"]]] = None,
        engine_mode: Optional[pulumi.Input[Union[str, "EngineMode"]]] = None,
        engine_version: Optional[pulumi.Input[str]] = None,
        final_snapshot_identifier: Optional[pulumi.Input[str]] = None,
        global_cluster_identifier: Optional[pulumi.Input[str]] = None,
        iam_database_authentication_enabled: Optional[pulumi.Input[bool]] = None,
        iam_roles: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
        iops: Optional[pulumi.Input[int]] = None,
        kms_key_id: Optional[pulumi.Input[str]] = None,
        manage_master_user_password: Optional[pulumi.Input[bool]] = None,
        master_password: Optional[pulumi.Input[str]] = None,
        master_user_secret_kms_key_id: Optional[pulumi.Input[str]] = None,
        master_username: Optional[pulumi.Input[str]] = None,
        network_type: Optional[pulumi.Input[str]] = None,
        port: Optional[pulumi.Input[int]] = None,
        preferred_backup_window: Optional[pulumi.Input[str]] = None,
        preferred_maintenance_window: Optional[pulumi.Input[str]] = None,
        replication_source_identifier: Optional[pulumi.Input[str]] = None,
        restore_to_point_in_time: Optional[pulumi.Input[pulumi.InputType["ClusterRestoreToPointInTimeArgs"]]] = None,
        s3_import: Optional[pulumi.Input[pulumi.InputType["ClusterS3ImportArgs"]]] = None,
        scaling_configuration: Optional[pulumi.Input[pulumi.InputType["ClusterScalingConfigurationArgs"]]] = None,
        serverlessv2_scaling_configuration: Optional[
            pulumi.Input[pulumi.InputType["ClusterServerlessv2ScalingConfigurationArgs"]]
        ] = None,
        skip_final_snapshot: Optional[pulumi.Input[bool]] = None,
        snapshot_identifier: Optional[pulumi.Input[str]] = None,
        source_region: Optional[pulumi.Input[str]] = None,
        storage_encrypted: Optional[pulumi.Input[bool]] = None,
        storage_type: Optional[pulumi.Input[str]] = None,
        tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
        vpc_security_group_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
        __props__=None,
    ):
        deletion_protection = deletion_protection if deletion_protection is not None else True
        manage_master_user_password = manage_master_user_password if manage_master_user_password is not None else True
        cluster_identifier = cluster_identifier if cluster_identifier is not None else name
        engine = engine if engine is not None else "aurora-postgresql"
        master_username = master_username if master_username is not None else "postgres"
        backup_retention_period = backup_retention_period if backup_retention_period is not None else 5
        iam_database_authentication_enabled = (
            iam_database_authentication_enabled if iam_database_authentication_enabled is not None else True
        )

        BaseResourceMixin.__init__(
            self,
            parent=rds.Cluster,
            resource_name=name,
            opts=opts,
            allocated_storage=allocated_storage,
            allow_major_version_upgrade=allow_major_version_upgrade,
            apply_immediately=apply_immediately,
            availability_zones=availability_zones,
            backtrack_window=backtrack_window,
            backup_retention_period=backup_retention_period,
            cluster_identifier=cluster_identifier,
            cluster_identifier_prefix=cluster_identifier_prefix,
            cluster_members=cluster_members,
            copy_tags_to_snapshot=copy_tags_to_snapshot,
            database_name=database_name,
            db_cluster_instance_class=db_cluster_instance_class,
            db_cluster_parameter_group_name=db_cluster_parameter_group_name,
            db_instance_parameter_group_name=db_instance_parameter_group_name,
            db_subnet_group_name=db_subnet_group_name,
            deletion_protection=deletion_protection,
            enable_global_write_forwarding=enable_global_write_forwarding,
            enable_http_endpoint=enable_http_endpoint,
            enabled_cloudwatch_logs_exports=enabled_cloudwatch_logs_exports,
            engine=engine,
            engine_mode=engine_mode,
            engine_version=engine_version,
            final_snapshot_identifier=final_snapshot_identifier,
            global_cluster_identifier=global_cluster_identifier,
            iam_database_authentication_enabled=iam_database_authentication_enabled,
            iam_roles=iam_roles,
            iops=iops,
            kms_key_id=kms_key_id,
            manage_master_user_password=manage_master_user_password,
            master_password=master_password,
            master_user_secret_kms_key_id=master_user_secret_kms_key_id,
            master_username=master_username,
            network_type=network_type,
            port=port,
            preferred_backup_window=preferred_backup_window,
            preferred_maintenance_window=preferred_maintenance_window,
            replication_source_identifier=replication_source_identifier,
            restore_to_point_in_time=restore_to_point_in_time,
            s3_import=s3_import,
            scaling_configuration=scaling_configuration,
            serverlessv2_scaling_configuration=serverlessv2_scaling_configuration,
            skip_final_snapshot=skip_final_snapshot,
            snapshot_identifier=snapshot_identifier,
            source_region=source_region,
            storage_encrypted=storage_encrypted,
            storage_type=storage_type,
            tags=tags,
            vpc_security_group_ids=vpc_security_group_ids,
            __props__=__props__,
        )

    def instance(
        self,
        name,
        opts: Optional[pulumi.ResourceOptions] = None,
        auto_minor_version_upgrade: Optional[pulumi.Input[bool]] = None,
        availability_zone: Optional[pulumi.Input[str]] = None,
        ca_cert_identifier: Optional[pulumi.Input[str]] = None,
        copy_tags_to_snapshot: Optional[pulumi.Input[bool]] = None,
        engine_version: Optional[pulumi.Input[str]] = None,
        identifier: Optional[pulumi.Input[str]] = None,
        identifier_prefix: Optional[pulumi.Input[str]] = None,
        instance_class: Optional[pulumi.Input[Union[str, "InstanceType"]]] = None,
        monitoring_interval: Optional[pulumi.Input[int]] = None,
        monitoring_role_arn: Optional[pulumi.Input[str]] = None,
        performance_insights_enabled: Optional[pulumi.Input[bool]] = None,
        performance_insights_kms_key_id: Optional[pulumi.Input[str]] = None,
        performance_insights_retention_period: Optional[pulumi.Input[int]] = None,
        preferred_backup_window: Optional[pulumi.Input[str]] = None,
        preferred_maintenance_window: Optional[pulumi.Input[str]] = None,
        promotion_tier: Optional[pulumi.Input[int]] = None,
        publicly_accessible: Optional[pulumi.Input[bool]] = None,
        tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
        __props__=None,
    ):
        return ClusterInstance(
            name=name,
            opts=opts,
            apply_immediately=self.apply_immediately,
            auto_minor_version_upgrade=auto_minor_version_upgrade,
            availability_zone=availability_zone,
            ca_cert_identifier=ca_cert_identifier,
            cluster_identifier=self.cluster_identifier,
            copy_tags_to_snapshot=copy_tags_to_snapshot,
            db_parameter_group_name=self.db_cluster_parameter_group_name,
            db_subnet_group_name=self.db_subnet_group_name,
            engine=self.engine,
            engine_version=engine_version,
            identifier=name,
            identifier_prefix=identifier_prefix,
            instance_class=instance_class,
            monitoring_interval=monitoring_interval,
            monitoring_role_arn=monitoring_role_arn,
            performance_insights_enabled=performance_insights_enabled,
            performance_insights_kms_key_id=performance_insights_kms_key_id,
            performance_insights_retention_period=performance_insights_retention_period,
            preferred_backup_window=preferred_backup_window,
            preferred_maintenance_window=preferred_maintenance_window,
            promotion_tier=promotion_tier,
            publicly_accessible=publicly_accessible,
            tags=tags,
        )


class ClusterInstance(rds.ClusterInstance, BaseResourceMixin):
    def __init__(
        self,
        name: str,
        opts: Optional[pulumi.ResourceOptions] = None,
        apply_immediately: Optional[pulumi.Input[bool]] = None,
        auto_minor_version_upgrade: Optional[pulumi.Input[bool]] = None,
        availability_zone: Optional[pulumi.Input[str]] = None,
        ca_cert_identifier: Optional[pulumi.Input[str]] = None,
        cluster_identifier: Optional[pulumi.Input[str]] = None,
        copy_tags_to_snapshot: Optional[pulumi.Input[bool]] = None,
        db_parameter_group_name: Optional[pulumi.Input[str]] = None,
        db_subnet_group_name: Optional[pulumi.Input[str]] = None,
        engine: Optional[pulumi.Input[str]] = None,
        engine_version: Optional[pulumi.Input[str]] = None,
        identifier: Optional[pulumi.Input[str]] = None,
        identifier_prefix: Optional[pulumi.Input[str]] = None,
        instance_class: Optional[pulumi.Input[Union[str, "InstanceType"]]] = None,
        monitoring_interval: Optional[pulumi.Input[int]] = None,
        monitoring_role_arn: Optional[pulumi.Input[str]] = None,
        performance_insights_enabled: Optional[pulumi.Input[bool]] = None,
        performance_insights_kms_key_id: Optional[pulumi.Input[str]] = None,
        performance_insights_retention_period: Optional[pulumi.Input[int]] = None,
        preferred_backup_window: Optional[pulumi.Input[str]] = None,
        preferred_maintenance_window: Optional[pulumi.Input[str]] = None,
        promotion_tier: Optional[pulumi.Input[int]] = None,
        publicly_accessible: Optional[pulumi.Input[bool]] = None,
        tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
        __props__=None,
    ):
        ca_cert_identifier = ca_cert_identifier if ca_cert_identifier is not None else "rds-ca-rsa2048-g1"
        BaseResourceMixin.__init__(
            self,
            resource_name=name,
            parent=rds.ClusterInstance,
            opts=opts,
            apply_immediately=apply_immediately,
            auto_minor_version_upgrade=auto_minor_version_upgrade,
            availability_zone=availability_zone,
            ca_cert_identifier=ca_cert_identifier,
            cluster_identifier=cluster_identifier,
            copy_tags_to_snapshot=copy_tags_to_snapshot,
            db_parameter_group_name=db_parameter_group_name,
            db_subnet_group_name=db_subnet_group_name,
            engine=engine,
            engine_version=engine_version,
            identifier=identifier,
            identifier_prefix=identifier_prefix,
            instance_class=instance_class,
            monitoring_interval=monitoring_interval,
            monitoring_role_arn=monitoring_role_arn,
            performance_insights_enabled=performance_insights_enabled,
            performance_insights_kms_key_id=performance_insights_kms_key_id,
            performance_insights_retention_period=performance_insights_retention_period,
            preferred_backup_window=preferred_backup_window,
            preferred_maintenance_window=preferred_maintenance_window,
            promotion_tier=promotion_tier,
            publicly_accessible=publicly_accessible,
            tags=tags,
        )


class Instance(rds.Instance, BaseResourceMixin):
    def __init__(
        self,
        name: str,
        opts: Optional[pulumi.ResourceOptions] = None,
        allocated_storage: Optional[pulumi.Input[int]] = None,
        allow_major_version_upgrade: Optional[pulumi.Input[bool]] = None,
        apply_immediately: Optional[pulumi.Input[bool]] = None,
        auto_minor_version_upgrade: Optional[pulumi.Input[bool]] = None,
        availability_zone: Optional[pulumi.Input[str]] = None,
        backup_retention_period: Optional[pulumi.Input[int]] = None,
        backup_target: Optional[pulumi.Input[str]] = None,
        backup_window: Optional[pulumi.Input[str]] = None,
        blue_green_update: Optional[pulumi.Input[pulumi.InputType["InstanceBlueGreenUpdateArgs"]]] = None,
        ca_cert_identifier: Optional[pulumi.Input[str]] = None,
        character_set_name: Optional[pulumi.Input[str]] = None,
        copy_tags_to_snapshot: Optional[pulumi.Input[bool]] = None,
        custom_iam_instance_profile: Optional[pulumi.Input[str]] = None,
        customer_owned_ip_enabled: Optional[pulumi.Input[bool]] = None,
        db_name: Optional[pulumi.Input[str]] = None,
        db_subnet_group_name: Optional[pulumi.Input[str]] = None,
        delete_automated_backups: Optional[pulumi.Input[bool]] = None,
        deletion_protection: Optional[pulumi.Input[bool]] = None,
        domain: Optional[pulumi.Input[str]] = None,
        domain_iam_role_name: Optional[pulumi.Input[str]] = None,
        enabled_cloudwatch_logs_exports: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
        engine: Optional[pulumi.Input[str]] = None,
        engine_version: Optional[pulumi.Input[str]] = None,
        final_snapshot_identifier: Optional[pulumi.Input[str]] = None,
        iam_database_authentication_enabled: Optional[pulumi.Input[bool]] = None,
        identifier: Optional[pulumi.Input[str]] = None,
        identifier_prefix: Optional[pulumi.Input[str]] = None,
        instance_class: Optional[pulumi.Input[Union[str, "InstanceType"]]] = None,
        iops: Optional[pulumi.Input[int]] = None,
        kms_key_id: Optional[pulumi.Input[str]] = None,
        license_model: Optional[pulumi.Input[str]] = None,
        maintenance_window: Optional[pulumi.Input[str]] = None,
        manage_master_user_password: Optional[pulumi.Input[bool]] = None,
        master_user_secret_kms_key_id: Optional[pulumi.Input[str]] = None,
        max_allocated_storage: Optional[pulumi.Input[int]] = None,
        monitoring_interval: Optional[pulumi.Input[int]] = None,
        monitoring_role_arn: Optional[pulumi.Input[str]] = None,
        multi_az: Optional[pulumi.Input[bool]] = None,
        nchar_character_set_name: Optional[pulumi.Input[str]] = None,
        network_type: Optional[pulumi.Input[str]] = None,
        option_group_name: Optional[pulumi.Input[str]] = None,
        parameter_group_name: Optional[pulumi.Input[str]] = None,
        password: Optional[pulumi.Input[str]] = None,
        performance_insights_enabled: Optional[pulumi.Input[bool]] = None,
        performance_insights_kms_key_id: Optional[pulumi.Input[str]] = None,
        performance_insights_retention_period: Optional[pulumi.Input[int]] = None,
        port: Optional[pulumi.Input[int]] = None,
        publicly_accessible: Optional[pulumi.Input[bool]] = None,
        replica_mode: Optional[pulumi.Input[str]] = None,
        replicate_source_db: Optional[pulumi.Input[str]] = None,
        restore_to_point_in_time: Optional[pulumi.Input[pulumi.InputType["InstanceRestoreToPointInTimeArgs"]]] = None,
        s3_import: Optional[pulumi.Input[pulumi.InputType["InstanceS3ImportArgs"]]] = None,
        skip_final_snapshot: Optional[pulumi.Input[bool]] = None,
        snapshot_identifier: Optional[pulumi.Input[str]] = None,
        storage_encrypted: Optional[pulumi.Input[bool]] = None,
        storage_throughput: Optional[pulumi.Input[int]] = None,
        storage_type: Optional[pulumi.Input[Union[str, "StorageType"]]] = None,
        tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
        timezone: Optional[pulumi.Input[str]] = None,
        username: Optional[pulumi.Input[str]] = None,
        vpc_security_group_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
        __props__=None,
    ):
        engine = engine if engine is not None else "aurora-postgresql"
        username = username if username is not None else "postgres"
        deletion_protection = deletion_protection if deletion_protection is not None else True
        manage_master_user_password = manage_master_user_password if manage_master_user_password is not None else True
        backup_retention_period = backup_retention_period if backup_retention_period is not None else 5
        allocated_storage = allocated_storage if allocated_storage is not None else 20

        BaseResourceMixin.__init__(
            self,
            parent=rds.Instance,
            resource_name=name,
            opts=opts,
            allocated_storage=allocated_storage,
            allow_major_version_upgrade=allow_major_version_upgrade,
            apply_immediately=apply_immediately,
            auto_minor_version_upgrade=auto_minor_version_upgrade,
            availability_zone=availability_zone,
            backup_retention_period=backup_retention_period,
            backup_target=backup_target,
            backup_window=backup_window,
            blue_green_update=blue_green_update,
            ca_cert_identifier=ca_cert_identifier,
            character_set_name=character_set_name,
            copy_tags_to_snapshot=copy_tags_to_snapshot,
            custom_iam_instance_profile=custom_iam_instance_profile,
            customer_owned_ip_enabled=customer_owned_ip_enabled,
            db_name=db_name,
            db_subnet_group_name=db_subnet_group_name,
            delete_automated_backups=delete_automated_backups,
            deletion_protection=deletion_protection,
            domain=domain,
            domain_iam_role_name=domain_iam_role_name,
            enabled_cloudwatch_logs_exports=enabled_cloudwatch_logs_exports,
            engine=engine,
            engine_version=engine_version,
            final_snapshot_identifier=final_snapshot_identifier,
            iam_database_authentication_enabled=iam_database_authentication_enabled,
            identifier=identifier,
            identifier_prefix=identifier_prefix,
            instance_class=instance_class,
            iops=iops,
            kms_key_id=kms_key_id,
            license_model=license_model,
            maintenance_window=maintenance_window,
            manage_master_user_password=manage_master_user_password,
            master_user_secret_kms_key_id=master_user_secret_kms_key_id,
            max_allocated_storage=max_allocated_storage,
            monitoring_interval=monitoring_interval,
            monitoring_role_arn=monitoring_role_arn,
            multi_az=multi_az,
            nchar_character_set_name=nchar_character_set_name,
            network_type=network_type,
            option_group_name=option_group_name,
            parameter_group_name=parameter_group_name,
            password=password,
            performance_insights_enabled=performance_insights_enabled,
            performance_insights_kms_key_id=performance_insights_kms_key_id,
            performance_insights_retention_period=performance_insights_retention_period,
            port=port,
            publicly_accessible=publicly_accessible,
            replica_mode=replica_mode,
            replicate_source_db=replicate_source_db,
            restore_to_point_in_time=restore_to_point_in_time,
            s3_import=s3_import,
            skip_final_snapshot=skip_final_snapshot,
            snapshot_identifier=snapshot_identifier,
            storage_encrypted=storage_encrypted,
            storage_throughput=storage_throughput,
            storage_type=storage_type,
            tags=tags,
            timezone=timezone,
            username=username,
            vpc_security_group_ids=vpc_security_group_ids,
            __props__=__props__,
        )


class SubnetGroup(rds.SubnetGroup, BaseResourceMixin):
    def __init__(
        self,
        name: str,
        subnet_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]],
        opts: Optional[pulumi.ResourceOptions] = None,
        description: Optional[pulumi.Input[str]] = None,
        name_prefix: Optional[pulumi.Input[str]] = None,
        tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
        __props__=None,
    ):
        BaseResourceMixin.__init__(
            self,
            resource_name=name,
            parent=rds.SubnetGroup,
            opts=opts,
            description=description,
            name_prefix=name_prefix,
            name=name,
            subnet_ids=subnet_ids,
            tags=tags,
            __props__=__props__,
        )

    def db_instance(self, name, vpc_id, shared_cidr, **kwargs):
        if not kwargs.get("kms_key_id", None):
            kms = Key(name)
            kms.create_alias(f"db-{name}")
            kwargs["kms_key_id"] = kms.arn
            kwargs["storage_encrypted"] = True

        kwargs.setdefault("instance_class", "db.t4g.medium")
        kwargs.setdefault("iops", 1000)
        # kwargs.setdefault("storage_type", "gp3")

        if not kwargs.get("vpc_security_group_ids", None):
            sg = SecurityGroup(
                name=f"db_{name}",
                vpc_id=vpc_id,
                egress=[
                    dict(
                        from_port=0,
                        to_port=0,
                        protocol="-1",
                        cidr_blocks=["0.0.0.0/0"],
                    )
                ],
                ingress=[
                    dict(
                        from_port=5432,
                        to_port=5432,
                        protocol="TCP",
                        cidr_blocks=[shared_cidr],
                    )
                ],
            )
            kwargs["vpc_security_group_ids"] = [sg.id]
            return Instance(name=name, db_subnet_group_name=self.name, **kwargs)

    def create_cluster(self, name, vpc_id, shared_cidr, instances=1, instance_class="db.t4g.medium", **kwargs):
        if not kwargs.get("kms_key_id", None):
            kms = Key(name)
            kms.create_alias(f"db-{name}")
            kwargs["kms_key_id"] = kms.arn
            kwargs["storage_encrypted"] = True

        kwargs.setdefault("backup_retention_period", 14)

        if not kwargs.get("vpc_security_group_ids", None):
            sg = SecurityGroup(
                name=f"db_{name}",
                vpc_id=vpc_id,
                egress=[
                    dict(
                        from_port=0,
                        to_port=0,
                        protocol="-1",
                        cidr_blocks=["0.0.0.0/0"],
                    )
                ],
                ingress=[
                    dict(
                        from_port=5432,
                        to_port=5432,
                        protocol="TCP",
                        cidr_blocks=[str(shared_cidr)],
                    )
                ],
            )
            kwargs["vpc_security_group_ids"] = [sg.id]

        cluster = Cluster(name, db_subnet_group_name=name, **kwargs)

        for i in range(0, instances):
            cluster.instance(name=f"cluster-{cluster.identifier.spinal}-instance-{i}", instance_class=instance_class)

        return cluster
