from typing import Optional, Mapping, overload, Sequence

from cloudlib.modules.resource import BaseResourceMixin
from pulumi_aws import networkfirewall
from cloudlib.utils.names import IdentifierV2
import pulumi


class RuleGroup(networkfirewall.RuleGroup, BaseResourceMixin):
    def __init__(
        self,
        name: Optional[pulumi.Input[str]],
        opts: Optional[pulumi.ResourceOptions] = None,
        capacity: Optional[pulumi.Input[int]] = None,
        description: Optional[pulumi.Input[str]] = None,
        encryption_configuration: Optional[
            pulumi.Input[pulumi.InputType["RuleGroupEncryptionConfigurationArgs"]]
        ] = None,
        rule_group: Optional[pulumi.Input[pulumi.InputType["RuleGroupRuleGroupArgs"]]] = None,
        rules: Optional[pulumi.Input[str]] = None,
        tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
        type: Optional[pulumi.Input[str]] = None,
        __props__=None,
        **kwargs
    ):
        BaseResourceMixin.__init__(
            self,
            parent=networkfirewall.RuleGroup,
            resource_name=name,
            opts=opts,
            capacity=capacity,
            description=description,
            encryption_configuration=encryption_configuration,
            name=IdentifierV2(name).spinal,
            rule_group=rule_group,
            rules=rules,
            tags=tags,
            type=type,
            __props__=__props__,
            **kwargs
        )


class Firewall(networkfirewall.Firewall, BaseResourceMixin):
    def __init__(
        self,
        name: Optional[pulumi.Input[str]],
        opts: Optional[pulumi.ResourceOptions] = None,
        delete_protection: Optional[pulumi.Input[bool]] = None,
        description: Optional[pulumi.Input[str]] = None,
        encryption_configuration: Optional[
            pulumi.Input[pulumi.InputType["FirewallEncryptionConfigurationArgs"]]
        ] = None,
        firewall_policy_arn: Optional[pulumi.Input[str]] = None,
        firewall_policy_change_protection: Optional[pulumi.Input[bool]] = None,
        subnet_change_protection: Optional[pulumi.Input[bool]] = None,
        subnet_mappings: Optional[
            pulumi.Input[Sequence[pulumi.Input[pulumi.InputType["FirewallSubnetMappingArgs"]]]]
        ] = None,
        tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
        vpc_id: Optional[pulumi.Input[str]] = None,
        __props__=None,
        **kwargs
    ):
        BaseResourceMixin.__init__(
            self,
            parent=networkfirewall.Firewall,
            resource_name=name,
            name=IdentifierV2(name).spinal,
            opts=opts,
            delete_protection=delete_protection,
            description=description,
            encryption_configuration=encryption_configuration,
            firewall_policy_arn=firewall_policy_arn,
            firewall_policy_change_protection=firewall_policy_change_protection,
            subnet_change_protection=subnet_change_protection,
            subnet_mappings=subnet_mappings,
            tags=tags,
            vpc_id=vpc_id,
            __props__=__props__,
            **kwargs
        )


class FirewallPolicy(networkfirewall.FirewallPolicy, BaseResourceMixin):
    def __init__(
        self,
        name: Optional[pulumi.Input[str]],
        opts: Optional[pulumi.ResourceOptions] = None,
        description: Optional[pulumi.Input[str]] = None,
        encryption_configuration: Optional[
            pulumi.Input[pulumi.InputType["FirewallPolicyEncryptionConfigurationArgs"]]
        ] = None,
        firewall_policy: Optional[pulumi.Input[pulumi.InputType["FirewallPolicyFirewallPolicyArgs"]]] = None,
        tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
        __props__=None,
        **kwargs
    ):
        BaseResourceMixin.__init__(
            self,
            parent=networkfirewall.FirewallPolicy,
            resource_name=name,
            name=IdentifierV2(name).spinal,
            opts=opts,
            description=description,
            encryption_configuration=encryption_configuration,
            firewall_policy=firewall_policy,
            tags=tags,
            __props__=__props__,
            **kwargs
        )
