from typing import Optional, Mapping, overload, Sequence

import pulumi
from pulumi_aws import ec2
from pulumi_aws.ec2 import (
    CustomerGatewayArgs,
    VpnGatewayArgs,
    VpnConnectionRouteInitArgs,
    VpnConnectionArgs,
    VpnGatewayRoutePropagationArgs,
)

from cloudlib.modules.resource import BaseResourceMixin


class CustomerGateway(ec2.CustomerGateway, BaseResourceMixin):
    def __init__(
        self,
        name: str,
        opts: Optional[pulumi.ResourceOptions] = None,
        bgp_asn: Optional[pulumi.Input[str]] = None,
        certificate_arn: Optional[pulumi.Input[str]] = None,
        device_name: Optional[pulumi.Input[str]] = None,
        ip_address: Optional[pulumi.Input[str]] = None,
        tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
        type: Optional[pulumi.Input[str]] = None,
        __props__=None,
        **kwargs,
    ):
        BaseResourceMixin.__init__(
            self,
            parent=ec2.CustomerGateway,
            resource_name=name,
            opts=opts,
            bgp_asn=bgp_asn or "65000",
            certificate_arn=certificate_arn,
            device_name=device_name,
            ip_address=ip_address,
            tags=tags,
            type=type,
            __props__=__props__,
            **kwargs,
        )


class VpnGateway(ec2.VpnGateway, BaseResourceMixin):
    def __init__(
        self,
        name: str,
        opts: Optional[pulumi.ResourceOptions] = None,
        amazon_side_asn: Optional[pulumi.Input[str]] = None,
        availability_zone: Optional[pulumi.Input[str]] = None,
        tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
        vpc_id: Optional[pulumi.Input[str]] = None,
        __props__=None,
        **kwargs,
    ):
        BaseResourceMixin.__init__(
            self,
            parent=ec2.VpnGateway,
            resource_name=name,
            opts=opts,
            amazon_side_asn=amazon_side_asn,
            availability_zone=availability_zone,
            tags=tags,
            vpc_id=vpc_id,
            __props__=__props__,
            **kwargs,
        )

    def propagate_routes(self, route_table_id):
        return VpnGatewayRoutePropagation(
            name=self.identifier.add("propagation"), route_table_id=route_table_id, vpn_gateway_id=self.id
        )


class VpnConnection(ec2.VpnConnection, BaseResourceMixin):
    def __init__(
        self,
        name: str,
        opts: Optional[pulumi.ResourceOptions] = None,
        customer_gateway_id: Optional[pulumi.Input[str]] = None,
        enable_acceleration: Optional[pulumi.Input[bool]] = None,
        local_ipv4_network_cidr: Optional[pulumi.Input[str]] = None,
        local_ipv6_network_cidr: Optional[pulumi.Input[str]] = None,
        outside_ip_address_type: Optional[pulumi.Input[str]] = None,
        remote_ipv4_network_cidr: Optional[pulumi.Input[str]] = None,
        remote_ipv6_network_cidr: Optional[pulumi.Input[str]] = None,
        static_routes_only: Optional[pulumi.Input[bool]] = None,
        tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
        transit_gateway_id: Optional[pulumi.Input[str]] = None,
        transport_transit_gateway_attachment_id: Optional[pulumi.Input[str]] = None,
        tunnel1_dpd_timeout_action: Optional[pulumi.Input[str]] = None,
        tunnel1_dpd_timeout_seconds: Optional[pulumi.Input[int]] = None,
        tunnel1_enable_tunnel_lifecycle_control: Optional[pulumi.Input[bool]] = None,
        tunnel1_ike_versions: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
        tunnel1_inside_cidr: Optional[pulumi.Input[str]] = None,
        tunnel1_inside_ipv6_cidr: Optional[pulumi.Input[str]] = None,
        tunnel1_log_options: Optional[pulumi.Input[pulumi.InputType["VpnConnectionTunnel1LogOptionsArgs"]]] = None,
        tunnel1_phase1_dh_group_numbers: Optional[pulumi.Input[Sequence[pulumi.Input[int]]]] = None,
        tunnel1_phase1_encryption_algorithms: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
        tunnel1_phase1_integrity_algorithms: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
        tunnel1_phase1_lifetime_seconds: Optional[pulumi.Input[int]] = None,
        tunnel1_phase2_dh_group_numbers: Optional[pulumi.Input[Sequence[pulumi.Input[int]]]] = None,
        tunnel1_phase2_encryption_algorithms: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
        tunnel1_phase2_integrity_algorithms: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
        tunnel1_phase2_lifetime_seconds: Optional[pulumi.Input[int]] = None,
        tunnel1_preshared_key: Optional[pulumi.Input[str]] = None,
        tunnel1_rekey_fuzz_percentage: Optional[pulumi.Input[int]] = None,
        tunnel1_rekey_margin_time_seconds: Optional[pulumi.Input[int]] = None,
        tunnel1_replay_window_size: Optional[pulumi.Input[int]] = None,
        tunnel1_startup_action: Optional[pulumi.Input[str]] = None,
        tunnel2_dpd_timeout_action: Optional[pulumi.Input[str]] = None,
        tunnel2_dpd_timeout_seconds: Optional[pulumi.Input[int]] = None,
        tunnel2_enable_tunnel_lifecycle_control: Optional[pulumi.Input[bool]] = None,
        tunnel2_ike_versions: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
        tunnel2_inside_cidr: Optional[pulumi.Input[str]] = None,
        tunnel2_inside_ipv6_cidr: Optional[pulumi.Input[str]] = None,
        tunnel2_log_options: Optional[pulumi.Input[pulumi.InputType["VpnConnectionTunnel2LogOptionsArgs"]]] = None,
        tunnel2_phase1_dh_group_numbers: Optional[pulumi.Input[Sequence[pulumi.Input[int]]]] = None,
        tunnel2_phase1_encryption_algorithms: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
        tunnel2_phase1_integrity_algorithms: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
        tunnel2_phase1_lifetime_seconds: Optional[pulumi.Input[int]] = None,
        tunnel2_phase2_dh_group_numbers: Optional[pulumi.Input[Sequence[pulumi.Input[int]]]] = None,
        tunnel2_phase2_encryption_algorithms: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
        tunnel2_phase2_integrity_algorithms: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
        tunnel2_phase2_lifetime_seconds: Optional[pulumi.Input[int]] = None,
        tunnel2_preshared_key: Optional[pulumi.Input[str]] = None,
        tunnel2_rekey_fuzz_percentage: Optional[pulumi.Input[int]] = None,
        tunnel2_rekey_margin_time_seconds: Optional[pulumi.Input[int]] = None,
        tunnel2_replay_window_size: Optional[pulumi.Input[int]] = None,
        tunnel2_startup_action: Optional[pulumi.Input[str]] = None,
        tunnel_inside_ip_version: Optional[pulumi.Input[str]] = None,
        type: Optional[pulumi.Input[str]] = None,
        vpn_gateway_id: Optional[pulumi.Input[str]] = None,
        __props__=None,
    ):
        BaseResourceMixin.__init__(
            self,
            parent=ec2.VpnConnection,
            resource_name=name,
            opts=opts,
            customer_gateway_id=customer_gateway_id,
            enable_acceleration=enable_acceleration,
            local_ipv4_network_cidr=local_ipv4_network_cidr,
            local_ipv6_network_cidr=local_ipv6_network_cidr,
            outside_ip_address_type=outside_ip_address_type,
            remote_ipv4_network_cidr=remote_ipv4_network_cidr,
            remote_ipv6_network_cidr=remote_ipv6_network_cidr,
            static_routes_only=static_routes_only,
            tags=tags,
            transit_gateway_id=transit_gateway_id,
            transport_transit_gateway_attachment_id=transport_transit_gateway_attachment_id,
            tunnel1_dpd_timeout_action=tunnel1_dpd_timeout_action,
            tunnel1_dpd_timeout_seconds=tunnel1_dpd_timeout_seconds,
            tunnel1_enable_tunnel_lifecycle_control=tunnel1_enable_tunnel_lifecycle_control,
            tunnel1_ike_versions=tunnel1_ike_versions,
            tunnel1_inside_cidr=tunnel1_inside_cidr,
            tunnel1_inside_ipv6_cidr=tunnel1_inside_ipv6_cidr,
            tunnel1_log_options=tunnel1_log_options,
            tunnel1_phase1_dh_group_numbers=tunnel1_phase1_dh_group_numbers,
            tunnel1_phase1_encryption_algorithms=tunnel1_phase1_encryption_algorithms,
            tunnel1_phase1_integrity_algorithms=tunnel1_phase1_integrity_algorithms,
            tunnel1_phase1_lifetime_seconds=tunnel1_phase1_lifetime_seconds,
            tunnel1_phase2_dh_group_numbers=tunnel1_phase2_dh_group_numbers,
            tunnel1_phase2_encryption_algorithms=tunnel1_phase2_encryption_algorithms,
            tunnel1_phase2_integrity_algorithms=tunnel1_phase2_integrity_algorithms,
            tunnel1_phase2_lifetime_seconds=tunnel1_phase2_lifetime_seconds,
            tunnel1_preshared_key=tunnel1_preshared_key,
            tunnel1_rekey_fuzz_percentage=tunnel1_rekey_fuzz_percentage,
            tunnel1_rekey_margin_time_seconds=tunnel1_rekey_margin_time_seconds,
            tunnel1_replay_window_size=tunnel1_replay_window_size,
            tunnel1_startup_action=tunnel1_startup_action,
            tunnel2_dpd_timeout_action=tunnel2_dpd_timeout_action,
            tunnel2_dpd_timeout_seconds=tunnel2_dpd_timeout_seconds,
            tunnel2_enable_tunnel_lifecycle_control=tunnel2_enable_tunnel_lifecycle_control,
            tunnel2_ike_versions=tunnel2_ike_versions,
            tunnel2_inside_cidr=tunnel2_inside_cidr,
            tunnel2_inside_ipv6_cidr=tunnel2_inside_ipv6_cidr,
            tunnel2_log_options=tunnel2_log_options,
            tunnel2_phase1_dh_group_numbers=tunnel2_phase1_dh_group_numbers,
            tunnel2_phase1_encryption_algorithms=tunnel2_phase1_encryption_algorithms,
            tunnel2_phase1_integrity_algorithms=tunnel2_phase1_integrity_algorithms,
            tunnel2_phase1_lifetime_seconds=tunnel2_phase1_lifetime_seconds,
            tunnel2_phase2_dh_group_numbers=tunnel2_phase2_dh_group_numbers,
            tunnel2_phase2_encryption_algorithms=tunnel2_phase2_encryption_algorithms,
            tunnel2_phase2_integrity_algorithms=tunnel2_phase2_integrity_algorithms,
            tunnel2_phase2_lifetime_seconds=tunnel2_phase2_lifetime_seconds,
            tunnel2_preshared_key=tunnel2_preshared_key,
            tunnel2_rekey_fuzz_percentage=tunnel2_rekey_fuzz_percentage,
            tunnel2_rekey_margin_time_seconds=tunnel2_rekey_margin_time_seconds,
            tunnel2_replay_window_size=tunnel2_replay_window_size,
            tunnel2_startup_action=tunnel2_startup_action,
            tunnel_inside_ip_version=tunnel_inside_ip_version,
            type=type,
            vpn_gateway_id=vpn_gateway_id,
            __props__=__props__,
        )


class VpnConnectionRoute(ec2.VpnConnectionRoute, BaseResourceMixin):
    def __init__(
        self,
        name: str,
        opts: Optional[pulumi.ResourceOptions] = None,
        destination_cidr_block: Optional[pulumi.Input[str]] = None,
        vpn_connection_id: Optional[pulumi.Input[str]] = None,
        __props__=None,
        **kwargs,
    ):
        BaseResourceMixin.__init__(
            self,
            parent=ec2.VpnConnectionRoute,
            resource_name=name,
            opts=opts,
            destination_cidr_block=destination_cidr_block,
            vpn_connection_id=vpn_connection_id,
            __props__=__props__,
            **kwargs,
        )


class VpnGatewayRoutePropagation(ec2.VpnGatewayRoutePropagation, BaseResourceMixin):
    def __init__(
        self,
        name: str,
        opts: Optional[pulumi.ResourceOptions] = None,
        route_table_id: Optional[pulumi.Input[str]] = None,
        vpn_gateway_id: Optional[pulumi.Input[str]] = None,
        __props__=None,
    ):
        BaseResourceMixin.__init__(
            self,
            resource_name=name,
            parent=ec2.VpnGatewayRoutePropagation,
            opts=opts,
            route_table_id=route_table_id,
            vpn_gateway_id=vpn_gateway_id,
            __props__=__props__,
        )
