import pulumi
from pulumi import Output

from cloudlib.utils.resources import ShareInfo


def get_shared_info(stage, region):
    stack = pulumi.StackReference(f"LivaNova/shared-networking/{stage}-{region}")

    x = Output.all(
        account_id=stack.get_output("account_id"),
        region_id=stack.get_output("region"),
        vpc_id=stack.get_output("vpc_id"),
        dns_rules=stack.get_output("dns_rules"),
        linked_manager_role=stack.get_output("linked_manager_role"),
        subnets_private=stack.get_output("subnets_private"),
        network=stack.get_output("network"),
        zone_id=stack.get_output("zone_id"),
        public_zone_id=stack.get_output("public_zone_id"),
        public_zone_name=stack.get_output("public_zone_name"),
        private_zone_id=stack.get_output("private_zone_id"),
        private_zone_name=stack.get_output("private_zone_name"),
    ).apply(lambda args: ShareInfo(stack=stack, **args))

    return x


def get_compute_info(stage, region):
    stack = pulumi.StackReference(f"LivaNova/shared-compute-core/{stage}-{region}")

    x = ShareInfo(
        stack=stack,
        lb_shared_arn=stack.get_output("lb_shared_arn"),
        lb_shared_listener_443=stack.get_output("lb_shared_listener_443"),
        lb_shared_security_group_id=stack.get_output("lb_shared_security_group_id"),
        lb_shared_zone_id=stack.get_output("lb_shared_zone_id"),
        lb_shared_dns_name=stack.get_output("lb_shared_dns_name"),
    )

    return x


class DictStackReference(pulumi.StackReference):
    def get(self, *args, **kwargs):
        return self.get_output(*args, **kwargs)


def get_core_networking(stage, region, *args):
    stack = DictStackReference(f"LivaNova/core-networking/{stage}-{region}")

    return stack
