import re

import pulumi
import inflection
from pulumi_aws import get_region


def domain(scope="epsy", **parts):
    config = pulumi.Config()
    domain_suffix = config.require("domain_suffix")
    scope = config.require_object(f"domains").get(scope)

    all_parts = list(parts.values()) + [scope, domain_suffix]

    return ".".join(filter(lambda x: x is not None, all_parts))


def regional_domain(scope="epsy", **parts):
    parts["region"] = get_region().name

    return domain(scope=scope, **parts)


class IdentifierV2(str):
    def __new__(cls, content):
        obj = super().__new__(cls, content)
        obj._content = content

        return obj

    @property
    def camel(self):
        return inflection.camelize(inflection.underscore(self._clean(self._content)))

    @property
    def pascal(self):
        return inflection.camelize(
            inflection.underscore(self._clean(self._content)), uppercase_first_letter=True
        ).replace("_", "")

    @property
    def snake(self):
        return inflection.underscore(self._clean(self._content))

    @property
    def spinal(self):
        return inflection.dasherize(inflection.underscore(self._clean(self._content)))

    @property
    def identifier(self):
        return re.sub(r"\W+", "_", self._content.lower()).strip("_")

    def add(self, content):
        return IdentifierV2(" ".join([self.strip(), content.strip()]).strip())

    def _clean(self, content):
        return re.sub(r"\W+", "_", content).strip("_")


class Identifier(str):
    def __new__(cls, content):
        return super().__new__(cls, re.sub(r"\W+", "_", content.lower()).strip("_"))

    def add(self, content):
        return Identifier("_".join([self, content]))
